<?php

namespace common\logic\Distribution\Distribution;

use common\helpers\GlobalParam;
use common\logic\BaseService;
use common\logic\Producer\Producer\Producer;
use common\logic\RepositoryInterface;

class DistributionRepository extends BaseService implements RepositoryInterface
{
    protected DistributionSolver $distributionSolver;

    public function __construct()
    {
        $this->distributionSolver = $this->loadService(DistributionSolver::class);
    }

    public function defaultOptionsSearch(): array
    {
        return [
            'with' => [],
            'join_with' => [],
            'orderby' => 'date ASC',
            'attribute_id_producer' => 'distribution.id_producer'
        ];
    }

    public function getOneById(int $id): ?Distribution
    {
        return Distribution::searchOne([
            'id' => $id
        ]);
    }

    public function getOne(Producer $producer, string $date): ?Distribution
    {
        $paramsDistribution = [
            'date' => $date,
            'distribution.id_producer' => $producer->id
        ];

        return Distribution::searchOne($paramsDistribution);
    }

    public function getOneByDate(string $date): ?Distribution
    {
        return Distribution::searchOne([
            'distribution.date' => $date
        ]);
    }

    /**
     * Récupère les distributions futures.
     */
    public function getIncoming(): array
    {
        $distributionsArray = Distribution::find()
            ->where('date > \'' . date('Y-m-d') . '\'')
            ->andWhere([
                'id_producer' => GlobalParam::getCurrentProducerId(),
                'active' => 1
            ])
            ->orderBy('date ASC')
            ->all();

        return $distributionsArray;
    }
    
    public function isDateAvailable(Producer $producer, string $date = null): bool
    {
        if($date && strlen($date)) {
            $distribution = $this->getOne($producer, $date);
            if($distribution && $this->distributionSolver->isAvailable($distribution)) {
                return true;
            }
        }
        
        return false;
    }
}