<?php

namespace common\logic\Producer\Producer;

use common\helpers\Opendistrib;
use common\helpers\Password;
use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\User\User\User;
use common\logic\User\UserProducer\UserProducer;
use common\logic\User\UserProducer\UserProducerBuilder;
use common\logic\User\UserProducer\UserProducerRepository;
use common\helpers\Url;

class ProducerBuilder extends BaseService implements BuilderInterface
{
    protected ProducerRepository $producerRepository;
    protected UserProducerRepository $userProducerRepository;
    protected UserProducerBuilder $userProducerBuilder;
    protected ProducerSolver $producerSolver;

    public function __construct()
    {
        $this->producerRepository = $this->loadService(ProducerRepository::class);
        $this->userProducerRepository = $this->loadService(UserProducerRepository::class);
        $this->userProducerBuilder = $this->loadService(UserProducerBuilder::class);
        $this->producerSolver = $this->loadService(ProducerSolver::class);
    }

    public function instanciateProducer(): Producer
    {
        $producer = new Producer();

        $producer->order_deadline = Producer::ORDER_DEADLINE_DEFAULT;
        $producer->order_delay = Producer::ORDER_DELAY_DEFAULT;

        return $producer;
    }

    public function init(Producer $producer): void
    {
        $this->initSlug($producer);
        $this->initCode($producer);
    }

    public function initSlug(Producer $producer): void
    {
        $cptSlug = 0 ;
        do {
            $slug = Url::slugify($producer->name) ;
            if($cptSlug) {
                $slug .= $cptSlug ;
            }
            $producer->slug = $slug ;
            $cptSlug ++ ;
        } while($this->producerRepository->getOneBySlug($producer->slug)) ;
    }

    public function initCode(Producer $producer): void
    {
        $producer->code = Password::generate();
    }

    /**
     * Lie un utilisateur à un producteur.
     */
    public function addUser(User $user, Producer $producer, int $bookmark = 1): UserProducer
    {
        $userProducer =  $this->userProducerBuilder->createIfNotExist($user, $producer, $bookmark);

        if (!$userProducer->getActive()) {
            $userProducer->setActive(1);
        }
        $userProducer->save();

        return $userProducer;
    }

    public function updateOpendistribVersion(Producer $producer): void
    {
        $versionsArray = Opendistrib::getVersions();
        $producer->latest_version_opendistrib = array_values($versionsArray)[0];
        $producer->save();
    }

    public function savePrivateKeyStripe($filename, $value)
    {
        if (strlen($value) > 0) {
            $handle = fopen($filename, "w");
            fwrite($handle, $value);
            fclose($handle);
        }
    }

    public function savePrivateKeyApiStripe(Producer $producer): void
    {
        $this->savePrivateKeyStripe(
            $this->producerSolver->getFilenamePrivateKeyApiStripe($producer),
            $producer->option_stripe_private_key
        );
    }

    public function savePrivateKeyEndpointStripe(Producer $producer): bool
    {
        $this->savePrivateKeyStripe(
            $this->producerSolver->getFilenamePrivateKeyEndpointStripe($producer),
            $producer->option_stripe_endpoint_secret
        );
    }
}