<?php

namespace common\logic\User\User;

use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\Producer\Producer\Producer;

class UserBuilder extends BaseService implements BuilderInterface
{
    public function instanciate(): User
    {
        $user = new User();

        return $user;
    }

    public function initPassword(User $user, string $password)
    {
        $user->setPassword($password);
        $user->generateAuthKey();
    }

    public function initProducer(User $user, Producer $producer)
    {
        $user->id_producer = $producer->id;
        $user->status = User::STATUS_PRODUCER;
    }

    /**
     * Met à jour la date de dernière connexion de l'utilisateur.
     */
    public function updateLastConnection(User $user)
    {
        $user->date_last_connection = date('Y-m-d H:i:s');
        $user->save();
    }

    /**
     * Generates password hash from password and sets it to the model
     *
     */
    public function setPassword(User $user, string $password): void
    {
        $user->password_hash = \Yii::$app->security->generatePasswordHash($password);
    }

    /**
     * Generates "remember me" authentication key
     */
    public function generateAuthKey(User $user): void
    {
        $user->auth_key = \Yii::$app->security->generateRandomString();
    }

    /**
     * Generates new password reset token
     */
    public function generatePasswordResetToken(User $user): void
    {
        $user->password_reset_token = \Yii::$app->security->generateRandomString() . '_' . time();
    }

    /**
     * Removes password reset token
     */
    public function removePasswordResetToken(User $user)
    {
        $user->password_reset_token = null;
    }

}